#!/usr/bin/env python
'''
    Contains the tools to check a contrib submission form
'''

import sys
#######################################################

try:
    from yaml import load, dump
    try:
        from yaml import CLoader as Loader, CDumper as Dumper
    except ImportError:
        from yaml import Loader, Dumper
except ImportError:
    print >> sys.stderr, "Could not import yaml lib"
    print >> sys.stderr, "Try installing PyYAML"
    raise ImportError('', "Cannot import yaml")

#######################################################



def check_template(filename):
    '''
        Verify that a contrib template YAML document is OK for
        submission
    '''
    template_versions = [ '1.0' ]

    _fd = open(filename, 'r')
    data = load(_fd, Loader=Loader)

    if not data.has_key('contrib_package_information'):
        raise ValueError('', "Missing contrib_package_information")

    template_version = str(data['contrib_package_information'])

    if template_version not in template_versions:
        raise ValueError('', "Invalid template version: " + template_version)

    licence = data['licence']
    package = data['package']
    package = data['explanation_of_hosting']
    production_ready = data['production_ready']
    if production_ready not in [ True, False ]:
        raise ValueError('', "production_ready not set correctly")

    try:
        check_maintainer(data, template_version)
    except:
        raise ValueError('', "Your template is incorrect fix submitted_by")

    try:
        check_project(data, template_version)
    except:
        raise ValueError('', "Your template is incorrect fix project info")

    try:
        check_release(data, template_version)
    except:
        raise ValueError('', "Your template is incorrect fix release info")

    try:
        check_fixes(data, template_version)
    except:
        raise ValueError('', "Your template is incorrect fix 'fixes'")

    try:
        check_rpms(data, template_version)
    except:
        raise ValueError('', "Your template is incorrect fix rpms")

    return 'OK'


def check_maintainer(data, template_version):
    '''
        Are the maintainer sections ok
    '''
    has_primary = False

    for contact in data['submitted_by']:
        name = contact['name']
        email = contact['email']
        primary_contact = contact['primary_contact']
        if primary_contact:
            if not has_primary:
                has_primary = primary_contact
            else:
                raise ValueError('', 'Only one Primary Contact allowed')

    if not has_primary:
        raise ValueError('', 'Missing Primary Contact')

def check_project(data, template_version):
    '''
        Is there information about upstream?
    '''
    project_page = data['project_page']
    project_code = data['project_code']

def check_release(data, template_version):
    '''
        release announcement data
    '''
    release_date = data['release_date']
    release_announcement = data['release_announcement']
    release_description = data['release_description']

def check_fixes(data, template_version):
    '''
        Check the 'fixes' stuff
    '''
    fixed_bugs = data['fixed_bugs']
    if fixed_bugs:
        for bug in fixed_bugs:
            url = bug['url']
            type = bug['type']
            id = bug['id']
            summary = bug['summary']

    fixed_cves = data['fixed_cves']
    security_update = data['security_update']
    if fixed_cves:
        if not security_update:
            raise ValueError('', 'Has fixed CVEs this is a security update')
        for cve in fixed_cves:
            id = cve['id']
            summary = cve['summary']

    if security_update not in [False, 'Critical', 'Important', 'Moderate', 'Low']:
        raise ValueError('', 'Bad security classification')
        

def check_rpms(data, template_version):
    '''
        Make sure the rpms are listed right
    '''
    rpms = data['rpms']
    for rpm in rpms:
        rpm_entries = rpm.keys()
        for entry in rpm_entries:
            if not entry.startswith('SL'):
                raise ValueError('', 'Bad rpm entry')
            suffix = entry.split('_',1)
            if suffix[1] not in ['SRPMS', 'i386', 'x86_64', 'debuginfo']:
                raise ValueError('', 'Bad rpm entry')

if __name__ == "__main__":

    if len(sys.argv) != 2:
        print >> sys.stderr, " You must provide a template to check"
        print >> sys.stderr, ""
        print >> sys.stderr, "  ie. " + sys.argv[0] + " <filename>"
        print >> sys.stderr, ""
        print >> sys.stderr, " This program just makes sure the template is"
        print >> sys.stderr, " complete, not that it will be accepted."
        sys.exit(1)

    RESULT = check_template(sys.argv[1])

    print RESULT
